################################################################################
################################################################################

### Computing metrics about Camera data
### By: Jono Handley, jonorow@gmail.com
### NOTE: As we learnt how to analyse the video data appropriately, we generated
### more than 1 R script. Hence, the script below is a synthesis of many scripts
### that were used toward the final analysis of the paper:

### "Behaviourally mediated predation avoidance in penguin prey: In situ evidence from animal-borne camera loggers"

################################################################################
################################################################################


################################################################################
### Set up environment
################################################################################

## Load required libraries
library(zoo)
library(xts) 
library(signal) ## For interpolation
library(lubridate) ## For time objects
library(plyr) ## To rename columns
library(data.table)

## Will adjust the number of decimal places displayed and help deal with split second issues
options(digits.secs=4)
#options(digits=0)

################################################################################
### Read in ALL merged CAM annotations from when the bird departed to sea
################################################################################

## Set you working directory
#setwd("~/PhD/WriteUp/WriteUp_Data Chapters/Chapter 5_Behaviour/Chap 5_Data/Cam/Cam_PreyInfo")
setwd("C:\\Users\\user\\Documents\\PhD\\WriteUp\\WriteUp_Data Chapters\\Chapter 5_Behaviour\\Chap 5_Supp Data and Code\\HandleyJM_2017_PenguinPrey_SuppData_RSOS")
#dir()

## load data file in
load("GNP_FLK_CAM_PreyDetail.Rdata")

## Check
head(d,2)
tail(d,2)

dim(d)

## NOTE: After annotating the videos 1st time around. A second set of annotations
## was done using the codes below which relate to the specific prey events.

table(d$PreyEvent)
## 1 = Prey event occured

table(d$PreyEventDetail)
## M = Munida/lobster krill; PC = Potential Capture; U = Unknown; SQI/SQ = Squid; F = Fish; LF = Larval Fish 

table(d$Capture)
## N = No; U = Unknown; FM = Mundia Fought Back; Y = Yes

table(d$LikelyPrey)
## LF = Larval Fish; M = munida/lobster krill; U = Unknown; SQ = Squid; MS = Munida off seabed; F = Fish; BB = Black Blob; TG = Themisto/Uknown

table(d$Phase)
## A = Ascend; B = bottom (head down); SW = swim water column; D = Descend; BH = Bottom (head up); SB = swim below surface

############################################################
### Calculate duration of each recording for each deployment
############################################################

head(d,2)
ListDep <- unique(d$Deployment)

trip.temp <- data.frame()
for(t in 1:length(ListDep)){
  time.data <- subset(d, d$Deployment==paste(ListDep[t]))
  time.data2 <- data.frame(time.data[1,c(1:3)], Duration_s = max(time.data$TripTime_s))
  trip.temp <- rbind(trip.temp,time.data2)
}

trip.temp

## Number of deployments at each site
table(trip.temp$Colony)

## Mean and SD of recording duration
(mean(trip.temp$Duration_s))/60
(sd(trip.temp$Duration_s))/60

## Total recording duration
((sum(trip.temp$Duration_s))/60)/60


##############################
## Create column of Unique feeding events
##############################

## Remove NA's from PreyEvent column
str(d)
d$PreyEvent[is.na(d$PreyEvent)] <- ""
str(d)

## Paste together unique combos of prey events and feeding detail
d$Feed <- paste(d$PreyEvent,d$PreyEventDetail,d$Capture,d$LikelyPrey,d$Phase, sep=" ")
table(d$Feed)
summary(d)
sum(as.numeric(d$PreyEvent))

## Look at only when feeding events occured
d2 <- subset(d, d$PreyEvent==1)
head(d2)
d2 <- droplevels(d2)
table(d2$Feed)
sum(table(d2$Feed))


## Determine the number of specific feeding events, rather than the number of frames (as above)
d$sep <- rleid(d$PreyEvent)
head(d,2)
dat1 <- subset(d, d$PreyEvent==1)
head(dat1,2)
dat1$sep2 <- rleid(dat1$sep)
head(dat1,2)

## Extract just the top row so you can look at individual feeding events
rm(temp.feed)
temp.feed <- data.frame()
for(i in 1:length(unique(dat1$sep2))){
  d <- subset(dat1, dat1$sep2== i)
  d2 <- d[1,]
  temp.feed <- rbind(temp.feed,d2)
}

head(temp.feed)
dim(temp.feed)

## Look at the feeding events
t <- table(temp.feed$Feed)
t
sum(t)
t2 <- data.frame(t)
t2

## Get range of APCs / PEEs for bird
length(unique(temp.feed$Deployment))
num <- table(temp.feed$Deployment)
num2 <- data.frame(num)

median(num2$Freq)
range(num2$Freq)
mean(num2$Freq)
sd(num2$Freq)
hist(num2$Freq,breaks=10)

## Save as a csv file for pivot table fun
write.csv(t2,file=paste("GNP_FLK_FeedEventsOverView.csv",sep=""),row.names=F)
write.csv(temp.feed,file=paste("GNP_FLK_Individual_FeedEventsOverView.csv",sep=""),row.names=F)

"With these CSV files I used pivot tables to determine metrics about feeding events"

############################################################
############################################################
############################################################
## Assess presence of munida on sea floor 
############################################################
############################################################
############################################################

load("GNP_FLK_CAM_PreyDetail.Rdata")

head(d,2)
tail(d,2)
table(d$Interspecific)

m.bed <- d 
m.bed$mf <- rleid(m.bed$Interspecific)
head(m.bed,2)

m.bed2 <- subset(m.bed, m.bed$Interspecific=="MunidaSeaBed_MS")
head(m.bed2)
m.bed2$mf2 <- rleid(m.bed2$mf)
max(m.bed2$mf2)

## and the duration of each of those events
head(m.bed2,2)

temp.mf <- data.frame()
for(m in 1: max(m.bed2$mf2)){
  mm <- subset(m.bed2, m.bed2$mf2==m)
  FeedDuration <- max(mm$TripTime_s)-min(mm$TripTime_s)
  mf.dat2 <- data.frame(mf2 = mm$mf2[1], FeedDuration_s = FeedDuration )
  ## rbind files together
  temp.mf <- rbind(temp.mf,mf.dat2) 
}

temp.mf

## Duration of events
mean(temp.mf$FeedDuration_s)
sd(temp.mf$FeedDuration_s)
median(temp.mf$FeedDuration_s)
range(temp.mf$FeedDuration_s)
hist(temp.mf$FeedDuration_s)



############################################################
############################################################
############################################################
## Assess presence of munida swarms
############################################################
############################################################
############################################################
load("GNP_FLK_CAM_PreyDetail.Rdata")

mund <- d

## Check unqiue factors in Interspecific column: Looking for, BaitBallMunida_BA
unique(mund$Interspecific)
head(mund,2)

## add a column stating when, BaitBallMunida_BA is present
mund$swarm <- ifelse(mund$Interspecific=="BaitBallMunida_BA",1,0)
head(mund,2)

## Use rleid function to make new number for each group of numbers in swarm column
mund$swamrmcount <- rleid(mund$swarm)
head(mund,2)
tail(mund,2)

## Extract only that part of the data frame relating to swarms
mund2 <- subset(mund, mund$Interspecific=="BaitBallMunida_BA")

## Use rleid function to make new number for each group of swarms
head(mund2,2)
mund2$swarm2 <- rleid(mund2$swamrmcount)
tail(mund2,2)
unique(mund2$Deployment)

## remove unused factor levels
mund2 <- droplevels(mund2)
unique(mund2$Deployment)

## extract the first row from each group of observations
mund3 <- mund2[!duplicated(mund2$swarm2),]
head(mund3,2)

## Save as csv
write.csv(mund3,file="MunidaSwarm_FrameRef.csv",row.names=F)

"With this CSV file, re-watch videos and assess interaction with munida swarms.
You can locate where in the video the swarms were present by the frame reference
number."


################################################################################
################################################################################
## INTER and INTRA events
################################################################################
################################################################################
load("GNP_FLK_CAM_PreyDetail.Rdata")

## Check
head(d,2)
tail(d,2)
dim(d)

####
table(d$Interspecific)

################################################################################
################################################################################
## Timing of interspecific events events
################################################################################
################################################################################

d.inter <- d

table(d.inter$Interspecific)
head(d.inter,2)

############################
## Interspecific
d.inter$UniqueInter <- rleid(d.inter$Interspecific)
## Subset out interspecific data
inter.data <- subset(d.inter, d.inter$Interspecific=="Roc Penguin_RP" | d.inter$Interspecific=="Mag Penguin_MP" | d.inter$Interspecific=="Penguin_PE")

head(inter.data)

inter.data <- droplevels(inter.data)
table(inter.data$Interspecific)

## Now apply the rleid function again BUT TO NEW COLUMMN
inter.data$UniqueInter2 <- rleid(inter.data$UniqueInter)
## Check 
head(inter.data,5)
max(inter.data$UniqueInter2)
## Loop over each feeding event
temp.inter <- data.frame()
for(m in 1: max(inter.data$UniqueInter2)){
  m.dat <- subset(inter.data, inter.data$UniqueInter2==m)
  print(m)
  InterDuration <- max(m.dat$TripTime_s)-min(m.dat$TripTime_s)
  m.dat2 <- data.frame(Deployment = m.dat$Deployment[1], Colony = m.dat$Colony[1], 
                       Interaction = m.dat$Interspecific[1], InterDuration_s = InterDuration,
                       PosWatColumn=m.dat$PosWatColumn[1])
  ## rbind files together
  temp.inter <- rbind(temp.inter,m.dat2) 
}

temp.inter


############################
## Intraspecific

d.intra <- d

table(d.intra$Intraspecific)

d.intra$UniqueIntra <- rleid(d.inter$Intraspecific)
## Subset out interspecific data
intra.data <- subset(d.intra, d.inter$Intraspecific!="Blank")

head(intra.data)

intra.data <- droplevels(intra.data)
table(intra.data$Intraspecific)

## Now apply the rleid function again BUT TO NEW COLUMMN
intra.data$UniqueIntra2 <- rleid(intra.data$UniqueIntra)
## Check 
head(intra.data,5)
max(intra.data$UniqueIntra2)
## Loop over each feeding event
temp.intra <- data.frame()
for(m in 1: max(intra.data$UniqueIntra2)){
  m.dat <- subset(intra.data, intra.data$UniqueIntra2==m)
  print(m)
  intraDuration <- max(m.dat$TripTime_s)-min(m.dat$TripTime_s)
  m.dat2 <- data.frame(Deployment = m.dat$Deployment[1], Colony = m.dat$Colony[1], 
                       Interaction = m.dat$Intraspecific[1], intraDuration_s = intraDuration,
                       PosWatColumn=m.dat$PosWatColumn[1])
  ## rbind files together
  temp.intra <- rbind(temp.intra,m.dat2) 
}

temp.intra
head(temp.intra)
summary(temp.intra)

temp.inter

############################
## Proportion on INTRA and INTER specific interaction time

trip.temp$DepN <- 1:nrow(trip.temp)
trip.temp$TrackDuration_mins <- trip.temp$Duration_s/60

dd <- data.frame()
for(i in 1:max(trip.temp$DepN)){
  d.temp <- subset(trip.temp, trip.temp$DepN==i)
  bird <- d.temp$Deployment[1]
  inter <- subset(temp.inter, temp.inter$Deployment==paste(bird))
  intra <- subset(temp.intra, temp.intra$Deployment==paste(bird))
  d2 <- data.frame(d.temp,InterDur_s = sum(inter$InterDuration_s), 
                   IntraDur_s = sum(intra$intraDuration_s), TrackDur_s=(d.temp$TrackDuration_mins*60) )
  dd <- rbind(dd,d2)
}

dd
head(dd,2)
dd$Inter_P <- (dd$InterDur_s/dd$TrackDur_s)*100
dd$Intra_P <- (dd$IntraDur_s/dd$TrackDur_s)*100
head(dd,2)

## Mean and SD (Inter) 
by(dd$Inter_P,dd$Colony,mean)
by(dd$Inter_P,dd$Colony,sd)

## Mean and SD (IntrA) 
by(dd$Intra_P,dd$Colony,mean)
by(dd$Intra_P,dd$Colony,sd)

