###############################################################################
###############################################################################

########       Annotated video data from Animal deployment       ##############
########                      jonorow@gmail.com                  ############## 
########      Example Script for basic data processing           ##############

###############################################################################
###############################################################################

#### About the study:
#### Video data was recorded from animal borne camera loggers placed on the
#### backs of Gentoo Penguins (Pygoscelis papua) at the Falkland Islands
#### Video was subsequently annotated frame by frame using the FREELY available
#### program: SOLOMON CODER. Video data came in batches of 30minute files (shortened for this example)
#### Therefore we need to merge and double check each of these files before analyses.

#### About the script
#### It has 3 parts, each with increasing detail


## Set you working directory: My folder contains only my CSV files
setwd("~/PhD/WriteUp/WriteUp_Data Chapters/Chapter 5_Behaviour/Chap 5_Supp Data and Code")

## Check what is inside
dir()

## Create a list of your file names
List <- dir(path=".",pattern="_Vid.csv",full.names=F)
List

##################
## PART 1
##################


## Read in the first file. Note  the use of skip to remove the first 4 lines of the csv file
i=3
data <- read.csv(List[i],header=T,sep=",", skip=4)

## View data
head(data,6)
str(data$Comments)
summary(data)

## See how many unique values you have for each column
sapply(data, function(x) length(unique(x)))
## See what those unique values are:
result <- sapply(data[,-1], function(x) unique(x))
result

## Need to remove the "" (blank) cell and the NA cells
## First the NA's
data[is.na(data)] <- "Blank"
## Then the blanks, which is easier to do after converting all columns to character
data[] <- lapply(data, as.character)
data[data==""]<-"Blank"
## Check
data
summary(data)
str(data)


## Extracting track information to append to data frame
name <- List[i]
name

Colony <- substr(name, ## This takes the name of your file
                 start=10, stop=12) ## Defines which part of the character string to take
Colony

Season <- substr(name, 
                 start=19, stop=22)
Season

Deployment <- substr(name, 
                     start=24, stop=30)
Deployment

Video <- substr(name, 
                start=32, stop=35)
Video

## Create new data frame with variables of interest
dataframe <- data.frame(Deployment=Deployment,
                        Colony=Colony,
                        Season=Season, Video=Video,
                        data)

dataframe$Frame <- 1:length(dataframe$Deployment)
head(dataframe)



################################################################################
################################################################################
################################################################################
################################################################################
################################################################################
################################################################################
################################################################################
################################################################################
################################################################################
################################################################################


##################
## PART 2
##################


### From here, loops and automates things a bit more. This is where you should start
### from for individual deployment merging. (See further in script for merging all files from all deployments)
rm(temp.data)
temp.data <- data.frame()
for(j in 1:3){
  data <- read.csv(List[j],header=T,sep=",", skip=4)
  ## Need to remove the "" (blank) cell and the NA cells
  ## First the NA's
  data[is.na(data)] <- "Blank"
  ## Then the blanks, which is easier to do after converting all columns to character
  data[] <- lapply(data, as.character)
  data[data==""]<-"Blank"
  ## Extracting track information to append to data frame
  name <- List[j]
  Colony <- substr(name,start=10, stop=12) 
  Season <- substr(name,start=19, stop=22)
  Deployment <- substr(name,start=24, stop=30)
  Video <- substr(name,start=32, stop=35)
  ## Create new data frame with variables of interest
  dataframe <- data.frame(Deployment=Deployment,
                          Colony=Colony,
                          Season=Season, Video=Video,
                          data, FrameActual = 1:nrow(data))
  ## Merge data frames
  temp.data <- rbind(temp.data,dataframe)
}

temp.data
head(temp.data,2)
tail(temp.data,2)

###############################################################################
## Get average frame speed: Note I say average because of technical
## issues with video data - read up on frame rate of 29.97 for further clarity.
###############################################################################

temp.data$Time <- as.numeric(temp.data$Time)

## First, create variable for subsetting each section of video
val <- unique(as.character((grep("1of",temp.data$Video,value=T))))
val2 <- unique(as.character((grep("2of",temp.data$Video,value=T))))
val3 <- unique(as.character((grep("3of",temp.data$Video,value=T))))
val4 <- unique(as.character((grep("4of",temp.data$Video,value=T))))
## Second, find the maximum value - hence the duration of that clip
maxval <- max(subset(temp.data,temp.data$Video==val)[,5])
maxval2 <- max(subset(temp.data,temp.data$Video==val2)[,5])
maxval3 <- max(subset(temp.data,temp.data$Video==val3)[,5])
maxval4 <- max(subset(temp.data,temp.data$Video==val4)[,5])
## Third, sum up the max durations to get total duration - use if else function
## because some videos came in batches of 3 vs 4.
if(maxval4=="-Inf"){
  TotalTrip <- sum(maxval,maxval2,maxval3)
} else {
  TotalTrip <- sum(maxval,maxval2,maxval3,maxval4)
}
## Fourth, get the average frame speed duration
frame.speed <- TotalTrip/nrow(temp.data)
frame.speed

####################################################
##### Now, create frame numbers for the entire trip.
####################################################
temp.data$FrameTotal <- 1:nrow(temp.data)
head(temp.data,2)
tail(temp.data,2)

################################################################################
## Now find the line in column Comments that corresponds to your reference time
## This was not always the first line as one had to wait for the start of a new second
################################################################################

## Pull out the start time
start.time <- grep("2013",temp.data$Comments,value=T)
start.time
class(start.time)
## Find the row number
time.begin <- which(grepl(start.time, temp.data$Comments))
time.begin

###########################################
## Convert start time into R friendly format
###########################################
start.time <- strptime(start.time, format = "%m/%d/%Y %H:%M:%S")
start.time <- as.POSIXct(start.time, tz = "GMT")
start.time
## Convert to epoch time (a useful reference number to use in coding without worrying about days,months,etc)
start.time.epoch <- as.numeric(as.POSIXct(start.time)) ## as.numeric will give you fractional seconds if you have :)
start.time.epoch

##########################################################################
### Subset out data to only inlcude that from when watch time was recorded
##########################################################################
temp.data <- temp.data[time.begin:nrow(temp.data),]
head(temp.data,2)
tail(temp.data,2)

############################################################
### Add new column relating each time recording to epochtime
############################################################

## First, create a new column from 1 to length of rows for easing conversion
temp.data$TimeConversion <- 1:nrow(temp.data) ## This will be same as FrameTotal if watch recording occured in first column

## Second, create time column for total elapsed trip time beginning from zero
temp.data$TimeTotal <- (temp.data$TimeConversion*frame.speed)-frame.speed
## Do some checking
check.val <- unique(as.character((grep("1of",temp.data$Video,value=T))))
checktime <- subset(temp.data,temp.data$Video==check.val)
head(checktime,2)
tail(checktime,2) ## The time values in columns, TimeTotal & Time, should match (although if you had frame rate 29.97 there might slight mismatch)
## Finally, add another column with total epochtime.
temp.data$TimeTotal.epoch <- temp.data$TimeTotal+start.time.epoch
## Some more checking
head(temp.data,2)
tail(temp.data,2) ## Hmm, you probably won't see the decimal places yet
## So change the base setting in the R session
options(digits=12)
## AND check again
head(temp.data,2)
tail(temp.data,2)

## Check time back from epochtime
temp.data$TimeActual <- as.POSIXct(temp.data$TimeTotal.epoch, origin = "1970-01-01", tz = "GMT")

### This will be entire footage information
save(temp.data, file=paste(Deployment,"_",Season,"_RawVid.Rdata",sep=''))

##############################################################
#### Extract the video information from when the bird departed 
##############################################################
options(digits=7)

## Find the row number relating to "Depart"
depart.row <- which(grepl("epart", temp.data$Comments)) # just in case I used D or d sometimes
depart.row
## Subset out the data
data.atsea <- temp.data[depart.row:nrow(temp.data),]
head(data.atsea,2)
tail(data.atsea,2)

##########################################
#### Add a column for time during the trip
##########################################
data.atsea$TripTimeConversion <- 1:nrow(data.atsea) 
data.atsea$TripTime_s <- (data.atsea$TripTimeConversion*frame.speed)-frame.speed
head(data.atsea,2)
tail(data.atsea,2)

### This should be pretty much the dataframe you need for analysis
save(data.atsea, file=paste(Deployment,"_",Season,"_VidAtSea.R",sep=''),row.names=F)


################################################################################
################################################################################
################################################################################
################################################################################
################################################################################
################################################################################
################################################################################
################################################################################
################################################################################
################################################################################

##################
## PART 3
##################


### From here, loops and automates things even further. This will loop over all deployments
### and create one single data file

## Need a temp data frame for storage
rm(d.temp)
d.temp <- data.frame()

ListDep <- unique(substr(List,start=24, stop=30))
ListDep
#ListBird <- grep(ListDep[1],List,value=T)
#ListBird

for(i in 1:length(ListDep)){
  ListBird <- grep(ListDep[i],List,value=T)
  ListBird
  rm(temp.data)
  temp.data <- data.frame()
      for(j in 1:length(ListBird)){
      data <- read.csv(ListBird[j],header=T,sep=",", skip=4)
      ## Need to remove the "" (blank) cell and the NA cells
      ## First the NA's
      data[is.na(data)] <- "Blank"
      ## Then the blanks, which is easier to do after converting all columns to character
      data[] <- lapply(data, as.character)
      data[data==""]<-"Blank"
      ## Extracting track information to append to data frame
      name <- ListBird[j]
      Colony <- substr(name,start=10, stop=12) 
      Season <- substr(name,start=19, stop=22)
      Deployment <- substr(name,start=24, stop=30)
      Video <- substr(name,start=32, stop=35)
      ## Create new data frame with variables of interest
      dataframe <- data.frame(Deployment=Deployment,
                              Colony=Colony,
                              Season=Season, Video=Video,
                              data, FrameActual = 1:nrow(data))
      ## Merge data frames
      temp.data <- rbind(temp.data,dataframe)
    }
    
    temp.data
    head(temp.data,2)
    tail(temp.data,2)
    
    ###############################################################################
    ## Get average frame speed: Note I say average because of technical
    ## issues with video data - read up on frame rate of 29.97 for further clarity.
    ###############################################################################
    
    temp.data$Time <- as.numeric(temp.data$Time)
    
    ## First, create variable for subsetting each section of video
    val <- unique(as.character((grep("1of",temp.data$Video,value=T))))
    val2 <- unique(as.character((grep("2of",temp.data$Video,value=T))))
    val3 <- unique(as.character((grep("3of",temp.data$Video,value=T))))
    val4 <- unique(as.character((grep("4of",temp.data$Video,value=T))))
    ## Second, find the maximum value - hence the duration of that clip
    maxval <- max(subset(temp.data,temp.data$Video==val)[,5])
    maxval2 <- max(subset(temp.data,temp.data$Video==val2)[,5])
    maxval3 <- max(subset(temp.data,temp.data$Video==val3)[,5])
    maxval4 <- max(subset(temp.data,temp.data$Video==val4)[,5])
    ## Third, sum up the max durations to get total duration - use if else function
    ## because some videos came in batches of 3 vs 4.
    if(maxval4=="-Inf"){
      TotalTrip <- sum(maxval,maxval2,maxval3)
    } else {
      TotalTrip <- sum(maxval,maxval2,maxval3,maxval4)
    }
    ## Fourth, get the average frame speed duration
    frame.speed <- TotalTrip/nrow(temp.data)
    frame.speed
    
    ####################################################
    ##### Now, create frame numbers for the entire trip.
    ####################################################
    temp.data$FrameTotal <- 1:nrow(temp.data)
    head(temp.data,2)
    tail(temp.data,2)
    
    ################################################################################
    ## Now find the line in column Comments that corresponds to your reference time
    ## This was not always the first line as one had to wait for the start of a new second
    ################################################################################
    
    ## Pull out the start time
    start.time <- grep("2013",temp.data$Comments,value=T)
    start.time
    class(start.time)
    ## Find the row number
    time.begin <- which(grepl(start.time, temp.data$Comments))
    time.begin
    
    ###########################################
    ## Convert start time into R friendly format
    ###########################################
    start.time <- strptime(start.time, format = "%m/%d/%Y %H:%M:%S")
    start.time <- as.POSIXct(start.time, tz = "GMT")
    start.time
    ## Convert to epoch time (a useful reference number to use in coding without worrying about days,months,etc)
    start.time.epoch <- as.numeric(as.POSIXct(start.time)) ## as.numeric will give you fractional seconds if you have :)
    start.time.epoch
    
    ##########################################################################
    ### Subset out data to only inlcude that from when watch time was recorded
    ##########################################################################
    temp.data <- temp.data[time.begin:nrow(temp.data),]
    head(temp.data,2)
    tail(temp.data,2)
    
    ############################################################
    ### Add new column relating each time recording to epochtime
    ############################################################
    
    ## First, create a new column from 1 to length of rows for easing conversion
    temp.data$TimeConversion <- 1:nrow(temp.data) ## This will be same as FrameTotal if watch recording occured in first column
    
    ## Second, create time column for total elapsed trip time beginning from zero
    temp.data$TimeTotal <- (temp.data$TimeConversion*frame.speed)-frame.speed
    ## Do some checking
    check.val <- unique(as.character((grep("1of",temp.data$Video,value=T))))
    checktime <- subset(temp.data,temp.data$Video==check.val)
    head(checktime,2)
    tail(checktime,2) ## The time values in columns, TimeTotal & Time, should match (although if you had frame rate 29.97 there might slight mismatch)
    ## Finally, add another column with total epochtime.
    temp.data$TimeTotal.epoch <- temp.data$TimeTotal+start.time.epoch
    ## Some more checking
    head(temp.data,2)
    tail(temp.data,2) ## Hmm, you probably won't see the decimal places yet
    ## So change the base setting in the R session
    options(digits=12)
    ## AND check again
    head(temp.data,2)
    tail(temp.data,2)
    
    ## Check time back from epochtime
    temp.data$TimeActual <- as.POSIXct(temp.data$TimeTotal.epoch, origin = "1970-01-01", tz = "GMT")
    
    ### This will be entire footage information
    #save(temp.data, file=paste(Deployment,"_",Season,"_RawVid.Rdata",sep=''))
    
    ##############################################################
    #### Extract the video information from when the bird departed 
    ##############################################################
    options(digits=7)
    
    ## Find the row number relating to "Depart"
    depart.row <- which(grepl("epart", temp.data$Comments)) # just in case I used D or d sometimes
    depart.row
    ## Subset out the data
    data.atsea <- temp.data[depart.row:nrow(temp.data),]
    head(data.atsea,2)
    tail(data.atsea,2)
    
    ##########################################
    #### Add a column for time during the trip
    ##########################################
    data.atsea$TripTimeConversion <- 1:nrow(data.atsea) 
    data.atsea$TripTime_s <- (data.atsea$TripTimeConversion*frame.speed)-frame.speed
    head(data.atsea,2)
    tail(data.atsea,2)
    
    d.temp <- rbind(d.temp,data.atsea)
}

head(d.temp,2)
tail(d.temp,2)
str(d.temp)

## See how many unique values you have for each behavioural category
sapply(d.temp[,6:16], function(x) length(unique(x)))
## See what those unique values are:
result <- sapply(d.temp[,6:16], function(x) unique(x))
result


### This should be largely the dataframe you need for analysis
save(d.temp, file=paste("AllTrips","_VidAtSea.R",sep=''),row.names=F)
write.csv(d.temp,file="AllTrips_VidAtSea.csv",row.names=F)

############################################################
### Calculate duration of each recording for each deployment
head(d.temp,2)

trip.temp <- data.frame()
for(t in 1:length(ListDep)){
  time.data <- subset(d.temp, d.temp$Deployment==paste(ListDep[t]))
  time.data2 <- data.frame(time.data[1,c(1:3)], Duration_s = max(time.data$TripTime_s))
  trip.temp <- rbind(trip.temp,time.data2)
}

trip.temp
