################################################################################
################################################################################

### Linking TDR and GPS data to Annotated (and Merged) CAMERA data
### By: Jono Handley, jonorow@gmail.com

### Note: This example is from actual data collected during the study

################################################################################
################################################################################

################################################################################
### Set up environment
################################################################################

## Load required libraries
library(zoo)
library(xts) 
library(signal) ## For interpolation
library(lubridate) ## For time objects
library(plyr) ## To rename columns
library(data.table)

## Will adjust the number of decimal places displayed and help deal with split second issues
options(digits.secs=4)
#options(digits=0)

## Set you working directory
setwd("~/PhD/WriteUp/WriteUp_Data Chapters/Chapter 5_Behaviour/Chap 5_Supp Data and Code")

#######################################
## Load your data file: CAM annotations
load("BR14_GNP_FLK_2013_CAM_SuppData.R")
## Check it
head(d4,2)
## rename for clarity
cam <- d4
rm(d4)

## Get unique naming variable
bird <- cam$Deployment[1]
bird

######################################
## Get the GPS data (This is processed data: erroneous points removed, interpolation via crawl package)
GPStrip <- dir(path=".",pattern=paste(bird,"_2013-14_GentooFI_bf_track",sep=""),full.names=F)
## Check
GPStrip
## Read the GPS file out
GPS <- read.csv(GPStrip,header=T,sep=",")
rm(GPStrip)

######################################
## Get the TDR data (This is processed data and has been zero-offset corrected: via diveMove package)
TDRtrip <- dir(path=".",pattern=paste(bird,"_TDR_2013.14_GentooFI_ZOCdives",sep=""),full.names=F)
## Check
TDRtrip
## Read the TDR file out
dive <- read.csv(TDRtrip,header=T,sep=",")
rm(TDRtrip)

######################################
## Check all files
head(cam,2)
head(GPS,2)
head(dive,2)
summary(dive)

######################################
## Convert time columns
dive$Timedive<-as.POSIXct(strptime(dive$Time, "%m/%d/%Y %H:%M:%S"), "GMT")
GPS$Timegps<-as.POSIXct(strptime(GPS$Time, "%m/%d/%Y %H:%M:%S"), "GMT")
cam$Timecam<-as.POSIXct(strptime(cam$TimeActual, "%Y-%m-%d %H:%M:%OS"), "GMT")



################################################################################
################################################################################
## Interpolate TDR and GPS data to CAM annotation

d.Pressure <- interp1(x=as.numeric(dive$Timedive), ## Time of data to be interpolated
                      y=dive$Depth, ## data to be interpolated
                      xi=as.numeric(cam$Timecam)) ## time of device you are interpolating to

d.Temp <- interp1(x=as.numeric(dive$Timedive), ## Time of data to be interpolated
                      y=dive$Temp, ## data to be interpolated
                      xi=as.numeric(cam$Timecam)) ## time of device you are interpolating to

d.Lon <- interp1(x=as.numeric(GPS$Timegps), ## Time of data to be interpolated
                  y=GPS$Lon, ## data to be interpolated
                  xi=as.numeric(cam$Timecam)) ## time of device you are interpolating to

d.Lat <- interp1(x=as.numeric(GPS$Timegps), ## Time of data to be interpolated
                 y=GPS$Lat, ## data to be interpolated
                 xi=as.numeric(cam$Timecam)) ## time of device you are interpolating to


d.Depth <- d.Pressure*-1


################################################################################
################################################################################
## Merge the cam data frame

cam2 <- data.frame(cam,Pressure=d.Pressure,Depth=d.Depth,Temp=d.Temp,Lon=d.Lon,lat=d.Lat)

## Check new data frame
head(cam2,2)



################################################################################
################################################################################
## Look at only when feeding events occured and plot

## specify: x coord, y coord, type = "l" means lines
plot(cam2$TripTime_s,cam2$Depth,
     #ylim = rev(range(tdrX$Pressure)),
     type="l",
     xlab="Time",ylab="Depth(m)")

## Determine the number of specific feeding events
cam2$sep <- rleid(cam2$PreyEvent)
head(cam2,2)
feed2 <- subset(cam2, cam2$PreyEvent==1)
head(feed2,2)
feed2$sep2 <- rleid(feed2$sep)
head(feed2,2)

## Extract just the top row so you can look at individual feeding events
rm(temp.feed)
temp.feed <- data.frame()
for(i in 1:length(unique(feed2$sep2))){
  z1 <- subset(feed2, feed2$sep2== i)
  z2 <- z1[1,]
  temp.feed <- rbind(temp.feed,z2)
}

head(temp.feed)
dim(temp.feed)

################################################################################
## Use the points function to add the points of interest from your data set 
## that you brought in
## Specifiy the x coord, y coord, type = bunch of choices here. See ?points
points(temp.feed$TripTime_s,temp.feed$Depth, type = "p", pch=16, cex=1, col=temp.feed$LikelyPrey) ## For dive from
text(temp.feed$TripTime_s,temp.feed$Depth, labels=temp.feed$sep2,pos=1)

title(bird)




